package com.gree.mall.contest.controller.mini.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.bean.order.*;
import com.gree.mall.contest.bean.pay.PayDetail;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.enums.QrCodeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.FreightLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.logic.order.OrderLogic;
import com.gree.mall.contest.logic.pay.PayLogic;
import com.gree.mall.contest.utils.IpUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Tag(name = "订单", description = "订单")
@RequestMapping(value = "/mini/order", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniOrderController {

    private final OrderLogic orderLogic;
    private final FreightLogic freightLogic;
    private final WechatLogic wechatLogic;
    private final PayLogic payLogic;
    private final CommonLogic commonLogic;
    private final RedisLockRegistry redisLockRegistry;


    @PostMapping("/ackdata")
    @Operation(summary = "确认订单内容")
    public ResponseHelper<OrderAckBean> ackData(
            @RequestBody OrderBuyBean orderBuyBean,
            HttpServletRequest request
    ) throws Exception {
        OrderAckBean orderAckBean = orderLogic.ackOrder(orderBuyBean,request);
        try {
            //运费
            if(orderAckBean.getFreight().doubleValue() < 0){
                return ResponseHelper.success(1100,orderAckBean,"暂不支持配送该地区");
            }
        }catch(Exception e){
            return ResponseHelper.success(1100,orderAckBean,e.getMessage());
        }
        return ResponseHelper.success(orderAckBean);
    }


    @PostMapping("/freight/v2")
    @Operation(summary = "获取运费")
    public ResponseHelper<BigDecimal> getFregiht(
            @RequestBody OrderBuyBean orderBuyBean,
            HttpServletRequest request
    ) throws RemoteServiceException {
        BigDecimal freightAmount = freightLogic.getFreightAmountV2(orderBuyBean,request);
        return ResponseHelper.success(freightAmount);
    }

    @PostMapping("/buy")
    @Operation(summary = "立即购买")
    public ResponseHelper<PayDetail> buy(
            @RequestBody OrderBuyBean orderBuyBean,
            HttpServletRequest request
    ) throws Exception {

        PayDetail buy = orderLogic.buy(orderBuyBean,request);
        return ResponseHelper.success(buy);
    }

    @PostMapping("/wait/pay")
    @Operation(summary = "支付待支付订单")
    public ResponseHelper<PayDetail> waitPayOrder(
            @Parameter(description = "支付人用户id") @RequestParam(required = false) String userId,
            @Parameter(description = "订单id", required = true) @RequestParam String orderId,
            @Parameter(description = "支付方式 微信支付 云闪付", required = true) @RequestParam(defaultValue = "微信支付") String payType,
            @Parameter(description = "购买人姓名") @RequestParam(required = false) String buyerName,
            @Parameter(description = "购买人身份证") @RequestParam(required = false) String buyerIdCard,
            HttpServletRequest request
    ) throws Exception {
        String ip = IpUtil.getIpAddr(request);
        PayDetail payDetail = orderLogic.waitPayOrder(userId, orderId, ip, payType, buyerName, buyerIdCard, request);
        return ResponseHelper.success(payDetail);
    }

    @GetMapping("/my/order")
    @Operation(summary = "我的订单")
    public ResponseHelper<IPage<OrderDetailBean>> myOrder(
            @Parameter(description = "用户id",required = true) @RequestParam String userId,
            @Parameter(description = "订单状态(NOPAY:待付款 DFH:待发货 YFH:已发货 OVER:已完成 CLOSE:已关闭)",required = false) @RequestParam(required = false) String orderStatus,
            @Parameter(description = "申请优惠状态 (100=全部  1=待审核 2=已通过 3=已驳回)",required = false) @RequestParam(required = false) Integer promotionApplyStatus,
            @Parameter(description = "pageNo",required = true) @RequestParam Integer pageNo,
            @Parameter(description = "pageSize",required = true) @RequestParam Integer pageSize
    ){
        IPage<OrderDetailBean> list = orderLogic.list(userId, orderStatus,promotionApplyStatus, pageNo, pageSize);
        return ResponseHelper.success(list);
    }

    @GetMapping("/detail")
    @Operation(summary = "订单详情")
    public ResponseHelper<OrderDetailBean> detail(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId
    ){
        OrderDetailBean detail = orderLogic.detail(orderId);
        return ResponseHelper.success(detail);
    }

    @GetMapping("/cancel")
    @Operation(summary = "取消订单")
    public ResponseHelper cancel(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId
    ) throws RemoteServiceException, InterruptedException {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + orderId);
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            throw new RemoteServiceException("请勿频繁操作，请稍后再试");
        }
        try {
            orderLogic.cancel(Collections.singletonList(orderId));
            return ResponseHelper.success();
        }finally {
            obtain.unlock();
        }
    }

    @PostMapping("/ack")
    @Operation(summary = "确认收货")
    public ResponseHelper ack(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId
    ) throws Exception {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + orderId);
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            throw new RemoteServiceException("请勿频繁操作，请稍后再试");
        }
        try {
            orderLogic.ack(orderId);
            return ResponseHelper.success();
        }finally {
            obtain.unlock();
        }
    }


    @PostMapping("/refund/cancel")
    @Operation(summary = "售后订单-撤销申请")
    public ResponseHelper<String> applyRefund(
            @Parameter(description = "订单id",required = true) @RequestParam String refundOrderId
    ) throws RemoteServiceException, InterruptedException {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + refundOrderId);
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            throw new RemoteServiceException("请勿频繁操作，请稍后再试");
        }
        try {
            String s = orderLogic.cancelRefund(refundOrderId);
            return ResponseHelper.success(s);
        }finally {
            obtain.unlock();
        }
    }


    @PostMapping("/refund/apply")
    @Operation(summary = "售后订单-申请退款")
    public ResponseHelper<String> applyRefund(
            @Parameter(description = "退款",required = true) @RequestBody RefundGoodsBean refundGoodsBean
    ) throws Exception {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + refundGoodsBean.getOrderId());
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            throw new RemoteServiceException("请勿频繁操作，请稍后再试");
        }
        try {
            String s = orderLogic.applyAfterSale(refundGoodsBean);
            return ResponseHelper.success(s);
        }finally {
            obtain.unlock();
        }
    }

    @GetMapping("/refund/list")
    @Operation(summary = "售后订单-列表")
    public ResponseHelper<IPage<OrderRefundDetailBean>> refundDetail(
            @Parameter(description = "用户id",required = true) @RequestParam String userId,
            @Parameter(description = "订单状态(DSJCL=待商家处理  DSJSH=待商家收货 DMJCL=待买家处理 OVER=已完成 CANCEL=已取消)",required = false) @RequestParam(required = false) String orderStatus,
            @Parameter(description = "pageNo",required = true) @RequestParam Integer pageNo,
            @Parameter(description = "pageSize",required = true) @RequestParam Integer pageSize
    ){
        IPage<OrderRefundDetailBean> orderRefundDetailBeanIPage = orderLogic.refundList(userId, orderStatus, pageNo, pageSize);
        return ResponseHelper.success(orderRefundDetailBeanIPage);
    }


    @GetMapping("/refund/detail")
    @Operation(summary = "售后订单-订单退款详情")
    public ResponseHelper<OrderRefundDetailBean> refundDetail(
            @Parameter(description = "售后订单id",required = true) @RequestParam String orderRefundId
    ){
        OrderRefundDetailBean orderRefundDetailBean = orderLogic.orderRefundDetail(orderRefundId);
        return ResponseHelper.success(orderRefundDetailBean);
    }

    @PostMapping("/refund/supply")
    @Operation(summary = "售后订单-商家已处理填写资料")
    public ResponseHelper applyRefund(
            @Parameter(description = "订单id",required = true) @RequestParam String orderRefundId,
            @Parameter(description = "物流单号",required = true) @RequestParam String logisticsNo,
            @Parameter(description = "快递公司code",required = true) @RequestParam String expressCompany,
            @Parameter(description = "凭证ids",required = true) @RequestParam List<String> imgIds
    ) throws RemoteServiceException, InterruptedException {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + orderRefundId);
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            throw new RemoteServiceException("请勿频繁操作，请稍后再试");
        }
        try {
            orderLogic.supplyData(orderRefundId, logisticsNo, expressCompany, imgIds);
            return ResponseHelper.success();
        }finally {
            obtain.unlock();
        }
    }

    @GetMapping("/count")
    @Operation(summary = "各订单状态的数量")
    public ResponseHelper<OrderStatusBean> count(
            @Parameter(description = "订单id",required = true) @RequestParam String userId
    ){
        OrderStatusBean count = orderLogic.count(userId);
        return ResponseHelper.success(count);
    }

    @GetMapping("/qrcode")
    @Operation(summary = "获取付费订单码")
    public ResponseHelper<String> qrcode(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId,
            @Parameter(description = "用户id",required = true) @RequestParam String userId,
            HttpServletRequest request
    ) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String qrcode = wechatLogic.getQrcode(QrCodeEnum.ORDER.toString().toLowerCase(Locale.ROOT),
                orderId,userId,currentCompanyWechat.getCompanyWechatId());
        return ResponseHelper.success(qrcode);
    }

    @PostMapping("/notice")
    @Operation(summary = "提醒发货")
    public ResponseHelper notice(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId
    ){
        orderLogic.notice(orderId);
        return ResponseHelper.success();
    }

}
